
import { createRoot } from 'react-dom/client';
import { StrictMode } from 'react';
import { BrowserRouter } from 'react-router-dom';
import App from './App.tsx';
import './index.css';
import './i18n'; // Import i18n configuration

// Performance measurement
if ('performance' in window && 'mark' in performance) {
  performance.mark('app-init-start');
}

// Global error handler with better reporting
window.addEventListener('error', (event) => {
  console.error('Global error caught:', event.error);
  
  // Prevent the browser from showing its own error dialog in production
  if (import.meta.env.PROD) {
    event.preventDefault();
  }
});

// Pre-connect to important domains for faster initial load
function setupPreconnects() {
  try {
    const domains = [
      'https://fonts.googleapis.com',
      'https://fonts.gstatic.com',
      'https://cdn.gpteng.co',
      'https://www.googletagmanager.com',
      'https://ouwtnbjowqhntgxmdvxt.supabase.co'
    ];
    
    domains.forEach(domain => {
      const link = document.createElement('link');
      link.rel = 'preconnect';
      link.href = domain;
      link.crossOrigin = 'anonymous';
      document.head.appendChild(link);
    });
  } catch (error) {
    console.error('Error setting up preconnects:', error);
    // Continue despite error - this is a performance optimization, not critical
  }
}

// Set up preconnects immediately
setupPreconnects();

// Use a more performance-focused approach to mounting
const rootElement = document.getElementById("root");

// Ensure root element exists
if (!rootElement) {
  throw new Error("Failed to find the root element");
}

// Function to render the app with appropriate error handling
function renderApp() {
  try {
    // Create root with error boundary
    const root = createRoot(rootElement);
    
    // Use StrictMode in development only for better performance in production
    if (import.meta.env.DEV) {
      root.render(
        <StrictMode>
          <BrowserRouter>
            <App />
          </BrowserRouter>
        </StrictMode>
      );
    } else {
      // In production, skip StrictMode for better performance
      root.render(
        <BrowserRouter>
          <App />
        </BrowserRouter>
      );
    }
    
    // Add performance metrics for monitoring
    if ('performance' in window && 'measure' in performance) {
      // Mark the end of initial load
      performance.mark('app-loaded');
      performance.measure('app-load-time', 'app-init-start', 'app-loaded');
    }
    
    return true;
  } catch (error) {
    console.error('Failed to render application:', error);
    
    // Render fallback UI in case of critical error
    try {
      const fallbackRoot = createRoot(rootElement);
      fallbackRoot.render(
        <div className="flex flex-col items-center justify-center min-h-screen bg-black text-white p-4">
          <h1 className="text-2xl font-bold mb-4">We're experiencing technical difficulties</h1>
          <p className="mb-6">Our team has been notified. Please try again later.</p>
          <button 
            onClick={() => window.location.reload()}
            className="px-4 py-2 bg-purple-600 text-white rounded hover:bg-purple-700"
          >
            Reload Application
          </button>
        </div>
      );
    } catch (fallbackError) {
      console.error('Failed to render fallback UI:', fallbackError);
    }
    
    return false;
  }
}

// Optimize rendering to not block the main thread
if (document.readyState === 'loading') {
  // If the document is still loading, wait for the DOMContentLoaded event
  document.addEventListener('DOMContentLoaded', () => {
    // Use requestIdleCallback to render the app when the browser is idle
    if ('requestIdleCallback' in window) {
      // @ts-ignore - TypeScript doesn't recognize requestIdleCallback
      window.requestIdleCallback(() => {
        renderApp();
      }, { timeout: 2000 }); // Fallback timeout of 2 seconds
    } else {
      // Fallback for browsers that don't support requestIdleCallback
      setTimeout(renderApp, 1);
    }
  });
} else {
  // Document already loaded, use requestIdleCallback immediately
  if ('requestIdleCallback' in window) {
    // @ts-ignore - TypeScript doesn't recognize requestIdleCallback
    window.requestIdleCallback(() => {
      renderApp();
    }, { timeout: 1000 }); // Shorter timeout since document is already loaded
  } else {
    // Fallback for browsers that don't support requestIdleCallback
    setTimeout(renderApp, 1);
  }
}
